@@ -10,8 +10,6 @@
 # mailto:mail@vipul.net (build a module based on Math::BigInt)
 # mailto:gary@hotlava.com (Math::BigInteger)
 
-# 2001-04-07 v1.23 Tels
- 
 # todo:
 # * fully remove funky $# stuff (maybe)
 # * use integer; vs 1e7 as base
@@ -19,6 +17,7 @@
 
 # Qs: what exactly happens on numify of HUGE numbers? overflow?
 #     $a = -$a is much slower (making copy of $a) than $a->bneg(), hm!?
+#     (copy_on_write will help there, but is still todo)
 
 # The following hash values are used:
 #   value: the internal array, base 100000
@@ -34,84 +33,78 @@
 package Math::BigInt;
 my $class = "Math::BigInt";
 
-$VERSION = 1.23;
+$VERSION = 1.31;
 use Exporter;
 @ISA =       qw( Exporter );
 @EXPORT_OK = qw( bneg babs bcmp badd bmul bdiv bmod bnorm bsub
-                 bgcd blcm 
+                 bgcd blcm
+		 bround 
                  blsft brsft band bior bxor bnot bpow bnan bzero 
-                 bacmp bstr binc bdec bint
+                 bacmp bstr bsstr binc bdec bint
                  is_odd is_even is_zero is_one is_nan sign
-		 length
+		 length as_number
 		 trace objectify _swap
                ); 
 
 #@EXPORT = qw( );
-use vars qw/$AUTOLOAD/;
+use vars qw/$AUTOLOAD $rnd_mode/;
 use strict;
 
-# Inside overload, the first arg is always an object. By using this, we can
-# determine from $_[0] the type of class. also, we can use clone() to make
-# a copy (since we can't modify it). The newly cloned object is used to call
-# the "right" type of calculation routine. Simple using badd() does not work
-# since then in a child the inheritance of badd() seems does not work like
-# expected due to closures inside overload.
-# When we don't need a clone(), we simple take ref() to get the type of class.
+# Inside overload, the first arg is always an object. If the original code had
+# it reversed (like $x = 2 * $y), then the third paramater indicates this
+# swapping. To make it work, we use a helper routine which not only reswaps the
+# params, but also makes a new object in this case. See _swap() for details,
+# especially the cases of operators with different classes.
 
-# The making of the second arg into an object is carried out by objectify,
-# and this determines the type of the class automatically, based on the first
-# argument's class.
-# The proper objectify (e.g. from a child class) is used if present,
-# Math::String uses this to make the second arg have the same charset than the
-# first.
+# For overloaded ops with only one argument we simple use $_[0]->copy() to
+# preserve the argument.
 
 # Thus inheritance of overload operators becomes possible and transparent for
-# our childs without the need to repeat the entire overload section there.
-
-sub clone;
+# our subclasses without the need to repeat the entire overload section there.
 
 use overload
-'='     =>      \&clone,
-#'+'	=>	sub { my @a = $_[0]->_swap(@_); $a[0]->badd($a[1]); },
-'+'	=>	sub { my $c = clone($_[0]); $c->badd($_[1]); },
+'='     =>      sub { $_[0]->copy(); },
 
-# sub is a bit tricky... b-a => -a+b
-'-'	=>	sub { my $c = clone($_[0]); $_[2] ?
+# '+' and '-' do not use _swap, since it is a triffle slower. If you want to
+# override _swap (if ever), then override overload of '+' and '-', too!
+# for sub it is a bit tricky to keep b: b-a => -a+b
+'-'	=>	sub { my $c = $_[0]->copy; $_[2] ?
                    $c->bneg()->badd($_[1]) :
                    $c->bsub( $_[1]) },
+'+'	=>	sub { $_[0]->copy()->badd($_[1]); },
 
-# some shortcuts for speed (assumes that reversed is routed to normal add etc)
+# some shortcuts for speed (assumes that reversed order of arguments is routed
+# to normal '+' and we thus can always modify first arg. If this is changed,
+# this breaks and must be adjusted.)
 '+='	=>	sub { $_[0]->badd($_[1]); },
 '-='	=>	sub { $_[0]->bsub($_[1]); },
 '*='	=>	sub { $_[0]->bmul($_[1]); },
 '/='	=>	sub { scalar $_[0]->bdiv($_[1]); },
 '**='	=>	sub { $_[0]->bpow($_[1]); },
 
-'<=>'	=>	sub { 
-			$_[2] ?
+'<=>'	=>	sub { $_[2] ?
                       $class->bcmp($_[1],$_[0]) : 
                       $class->bcmp($_[0],$_[1])},
 'cmp'	=>	sub { 
-	#my $c = ref($_[0]); 
          $_[2] ? 
                $_[1] cmp $_[0]->bstr() :
                $_[0]->bstr() cmp $_[1] },
 
-# dont need 'int' here ;)
-'neg'	=>	sub { my $c = clone($_[0]); $c->bneg(); }, 
-'abs'	=>	sub { my $c = clone($_[0]); $c->babs(); },
+'int'	=>	sub { $_[0]->copy(); }, 
+'neg'	=>	sub { $_[0]->copy()->bneg(); }, 
+'abs'	=>	sub { $_[0]->copy()->babs(); },
+'~'	=>	sub { $_[0]->copy()->bnot(); },
 
-'*'	=>	sub { my @a = $_[0]->_swap(@_); $a[0]->bmul($a[1]); },
-'/'	=>	sub { my @a = $_[0]->_swap(@_); scalar $a[0]->bdiv($a[1]); },
-'%'	=>	sub { my @a = $_[0]->_swap(@_); $a[0]->bmod($a[1]); },
-'**'	=>	sub { my @a = $_[0]->_swap(@_); $a[0]->bpow($a[1]); },
-'<<'	=>	sub { my @a = $_[0]->_swap(@_); $a[0]->blsft($a[1]); },
-'>>'	=>	sub { my @a = $_[0]->_swap(@_); $a[0]->brsft($a[1]); },
+'*'	=>	sub { my @a = ref($_[0])->_swap(@_); $a[0]->bmul($a[1]); },
+'/'	=>	sub { my @a = ref($_[0])->_swap(@_);scalar $a[0]->bdiv($a[1]);},
+'%'	=>	sub { my @a = ref($_[0])->_swap(@_); $a[0]->bmod($a[1]); },
+'**'	=>	sub { my @a = ref($_[0])->_swap(@_); $a[0]->bpow($a[1]); },
+'<<'	=>	sub { my @a = ref($_[0])->_swap(@_); $a[0]->blsft($a[1]); },
+'>>'	=>	sub { my @a = ref($_[0])->_swap(@_); $a[0]->brsft($a[1]); },
 
-'&'	=>	sub { my $c = clone($_[0]); $c->band($_[1]); },
-'|'	=>	sub { my $c = clone($_[0]); $c->bior($_[1]); },
-'^'	=>	sub { my $c = clone($_[0]); $c->bxor($_[1]); },
-'~'	=>	sub { my $c = clone($_[0]); $c->bnot(); },
+'&'	=>	sub { my @a = ref($_[0])->_swap(@_); $a[0]->band($a[1]); },
+'|'	=>	sub { my @a = ref($_[0])->_swap(@_); $a[0]->bior($a[1]); },
+'^'	=>	sub { my @a = ref($_[0])->_swap(@_); $a[0]->bxor($a[1]); },
 
 # can modify arg of ++ and --, so avoid a new-copy for speed, but don't
 # use $_[0]->_one(), it modifies $_[0] to be 1!
@@ -121,7 +114,10 @@ use overload
 # if overloaded, O(1) instead of O(N) and twice as fast for small numbers
 'bool'  =>	sub {
   # this kludge is needed for perl prior 5.6.0 since returning 0 here fails :-/
-  return !$_[0]->is_zero() || undef;
+  # v5.6.1 dumps on that: return !$_[0]->is_zero() || undef;		    :-(
+  my $t = !$_[0]->is_zero();
+  undef $t if $t == 0;
+  return $t;
   },
 
 qw(
@@ -135,28 +131,21 @@ my $NaNOK=1;
 my $trace = 0;
 # constant for easier life
 my $nan = 'NaN'; 
-my $BASE = 1e5;
+my $BASE = 1e5;			# var for trying to change it to 1e7
+my $RBASE = 1e-5;		# see USE_MUL
+
+# Rounding modes one of 'even', 'odd', '+inf', '-inf', 'zero' or 'trunc'
+$rnd_mode = 'even';
 
 ##############################################################################
 # constructors
 
-sub clone
-  {
-  # call the correct sub (due to closures), may be removed later on for speed
-  #$trace = 1;
-  #trace(@_);
-  #$trace = 0;
-  # this is wrong since it does not respect inheritance, ouch!
-  my $x = shift;
-  return $x->copy() if ref($x);
-  $class->new( $x ); 
-  }
-
 sub copy
   {
   my ($c,$x);
   if (@_ > 1)
     {
+    # if two arguments, the first one is the class to "swallow" subclasses
     ($c,$x) = @_;
     }
   else
@@ -184,7 +173,7 @@ sub copy
     elsif (ref($x->{$k}))
       {
       my $c = ref($x->{$k});
-      $self->{$k} = $c->new($x->{$k}); #->new(); # no copy() due to deep rec
+      $self->{$k} = $c->new($x->{$k}); # no copy() due to deep rec
       }
     else
       {
@@ -244,7 +233,7 @@ sub new
       }
     else
       {
-      if ($$mfv != 0)				# e <= 0
+      if ($$mfv ne '')				# e <= 0
         {
         # fraction and negative/zero E => NOI
         #print "NOI 2 \$\$mfv '$$mfv'\n";
@@ -313,6 +302,21 @@ sub bzero
 ##############################################################################
 # string conversation
 
+sub bsstr
+  {
+  # (ref to BFLOAT or num_str ) return num_str
+  # Convert number from internal format to scientific string format.
+  # internal format is always normalized (no leading zeros, "-0E0" => "+0E0")
+  trace(@_);
+  my ($self,$x) = objectify(1,@_);
+
+  my ($m,$e) = $x->parts();
+  # can be only '+', so
+  my $sign = 'e+';	
+  # MBF: my $s = $e->{sign}; $s = '' if $s eq '-'; my $sep = 'e'.$s;
+  return $m->bstr().$sign.$e->bstr();
+  }
+
 sub bstr 
   {
   # (ref to BINT or num_str ) return num_str
@@ -353,7 +357,7 @@ sub numify
 
   return $nan if $x->{sign} eq $nan;
   my $fac = 1; $fac = -1 if $x->{sign} eq '-';
-  return $fac*$x->{value}->[0] if @{$x->{value}} == 1;	# below 1e5
+  return $fac*$x->{value}->[0] if @{$x->{value}} == 1;	# below $BASE
   my $num = 0;
   foreach (@{$x->{value}})
     {
@@ -441,8 +445,8 @@ sub badd
   return $x->bnan() if (($x->{sign} eq $nan) || ($y->{sign} eq $nan));
 
   # speed: no add for 0+y or x+0
-  return $x if $y->is_zero(); # is0($y->{value},$y->{sign});       # x+0
-  if ($x->is_zero()) # is0($x->{value},$x->{sign}))                # 0+y
+  return $x if $y->is_zero(); # is0($y->{value},$y->{sign});	# x+0
+  if ($x->is_zero())						# 0+y
     {
     # make copy, clobbering up x
     $x->{value} = [ @{$y->{value}} ];
@@ -534,12 +538,16 @@ sub bgcd
   { 
   # (BINT or num_str, BINT or num_str) return BINT
   # does not modify arguments, but returns new object
-  # GCD -- Euclids algorithm Knuth Vol 2 pg 296
+  # GCD -- Euclids algorithm, variant C (Knuth Vol 3, pg 341 ff)
   trace(@_);
-   
+  
   my ($self,@arg) = objectify(0,@_);
-  my $x = $self->new(shift @arg);
-  while (@arg) { $x = _gcd($x,shift @arg); } 
+  my $x = $self->new(shift @arg); 
+  while (@arg)
+    {
+    #$x = _gcd($x,shift @arg); last if $x->is_one(); # new fast, but is slower
+    $x = _gcd_old($x,shift @arg); last if $x->is_one();	# old, slow, but faster
+    } 
   $x;
   }
 
@@ -614,8 +622,9 @@ sub bmul
   # multiply two numbers -- stolen from Knuth Vol 2 pg 233
   # (BINT or num_str, BINT or num_str) return BINT
   my ($self,$x,$y) = objectify(2,@_);
+  #print "$self bmul $x ",ref($x)," $y ",ref($y),"\n";
   trace(@_);
-  return $x if $x->modify('bdiv');
+  return $x if $x->modify('bmul');
   return $x->bnan() if (($x->{sign} eq $nan) || ($y->{sign} eq $nan));
 
   mul($x,$y);  # do actual math
@@ -666,6 +675,8 @@ sub bdiv
   my $r = $self->bzero(); 
   $r->{sign} = $y->{sign};
   ($x->{value},$r->{value}) = div($x->{value},$y->{value});
+  # do not leave rest "-0";
+  $r->{sign} = '+' if (@{$r->{value}} == 1) && ($r->{value}->[0] == 0);
   if (($x->{sign} eq '-') and (!$r->is_zero()))
     {
     $x->bdec();
@@ -766,7 +777,7 @@ sub band
   return $x->bnan() if ($x->{sign} eq $nan || $y->{sign} eq $nan);
   my $r = $self->bzero(); my $m = new Math::BigInt 1; my ($xr,$yr);
   my $x10000 = new Math::BigInt (0x10000);
-  my $y1 = clone($y);		 		# make copy
+  my $y1 = copy(ref($x),$y);		 		# make copy
   while (!$x->is_zero() && !$y1->is_zero())
     {
     ($x, $xr) = bdiv($x, $x10000);
@@ -788,7 +799,7 @@ sub bior
   return $x->bnan() if ($x->{sign} eq $nan || $y->{sign} eq $nan);
   my $r = $self->bzero(); my $m = new Math::BigInt 1; my ($xr,$yr);
   my $x10000 = new Math::BigInt (0x10000);
-  my $y1 = $y->clone();		 		# make copy
+  my $y1 = copy(ref($x),$y);		 		# make copy
   while (!$x->is_zero() || !$y1->is_zero())
     {
     ($x, $xr) = bdiv($x,$x10000);
@@ -810,7 +821,7 @@ sub bxor
   return $x->bzero() if $x == $y; # shortcut
   my $r = $self->bzero(); my $m = new Math::BigInt 1; my ($xr,$yr);
   my $x10000 = new Math::BigInt (0x10000);
-  my $y1 = clone($y);		 		# make copy
+  my $y1 = copy(ref($x),$y);	 		# make copy
   while (!$x->is_zero() || !$y1->is_zero())
     {
     ($x, $xr) = bdiv($x, $x10000);
@@ -823,8 +834,9 @@ sub bxor
 
 sub length
   {
-  trace(@_);
   my ($self,$x) = objectify(1,@_);
+
+  return (_digits($x->{value}), 0) if wantarray;
   _digits($x->{value});
   }
 
@@ -846,6 +858,203 @@ sub digit
   return substr($elem,-$digit-1,1);
   }
 
+sub _trailing_zeros
+  {
+  # return the amount of trailing zeros
+  my $x = shift;
+  $x = $class->new($x) unless ref $x;
+
+  return 0 if $x->is_zero() || $x->is_nan();
+  # check each array elem in _m for having 0 at end as long as elem == 0
+  # Upon finding a elem != 0, stop
+  my $zeros = 0; my $elem;
+  foreach my $e (@{$x->{value}})
+    {
+    if ($e != 0)
+      {
+      $elem = "$e";				# preserve x
+      $elem =~ s/.*?(0*$)/$1/;			# strip anything not zero
+      $zeros += CORE::length($elem);		# count trailing zeros
+      last;					# early out
+      }
+    else
+      {
+      $zeros += 5;				# all zeros
+      }
+    }
+  return $zeros;
+  }
+
+sub bsqrt
+  {
+  my ($self,$x) = objectify(1,@_);
+
+  return $x->bnan() if $x->{sign} =~ /\-|$nan/;	# -x or NaN => NaN
+  return $x->bzero() if $x->is_zero();		# 0 => 0
+  return $x if $x == 1;				# 1 => 1
+
+  my $y = $x->copy();				# give us one more digit accur.
+  my $l = int($x->length()/2);
+  
+  $x->bzero(); 
+  $x->binc();		# keep ref($x), but modify it
+  $x *= 10 ** $l;
+
+  # print "x: $y guess $x\n";
+
+  my $last = $self->bzero();
+  while ($last != $x)
+    {
+    $last = $x; 
+    $x += $y / $x; 
+    $x /= 2;
+    }
+  return $x;
+  }
+
+sub exponent
+  {
+  # return a copy of the exponent (here always 0, NaN or 1 for $m == 0)
+  my ($self,$x) = objectify(1,@_);
+ 
+  return bnan() if $x->is_nan();
+  my $e = $class->bzero();
+  return $e->binc() if $x->is_zero();
+  $e += $x->_trailing_zeros();
+  return $e;
+  }
+
+sub mantissa
+  {
+  # return a copy of the mantissa (here always $self)
+  my ($self,$x) = objectify(1,@_);
+
+  return bnan() if $x->is_nan();
+  my $m = $x->copy();
+  # that's inefficient
+  my $zeros = $m->_trailing_zeros();
+  $m /= 10 ** $zeros if $zeros != 0;
+  return $m;
+  }
+
+sub parts
+  {
+  # return a copy of both the exponent and the mantissa (here 0 and self)
+  my $self = shift;
+  $self = $class->new($self) unless ref $self;
+
+  return ($self->mantissa(),$self->exponent());
+  }
+   
+##############################################################################
+# rounding functions
+
+sub bfround
+  {
+  # precision: round to the $Nth digit left (+$n) or right (-$n) from the '.'
+  # $n == 0 => round to integer
+  my $x = shift; $x = $class->new($x) unless ref $x;
+  my $scale = shift || 0;
+  my $mode = shift || $rnd_mode;
+
+  # no-op for BigInts if $n <= 0
+  return $x if $scale <= 0;
+
+  $x->bround( $x->length()-$scale, $mode);
+  }
+
+sub _scan_for_nonzero
+  {
+  my $x = shift;
+  my $pad = shift;
+ 
+  my $len = $x->length();
+  return 0 if $len == 1;		# '5' is trailing by invisible zeros
+  my $follow = $pad - 1;
+  return 0 if $follow > $len || $follow < 1;
+  my $r = substr ("$x",-$follow);
+  # print "scan $x $pad $r\n";
+  return 1 if $r =~ /[^0]/;
+  return 0;
+  }
+
+sub fround
+  {
+  # to make life easier for switch between MBF and MBI (autoload like MBF?)
+  my $x = shift;
+  return $x->bround(@_);
+  }
+
+sub bround
+  {
+  # accuracy: +$n preserve $n digits from left,
+  #           -$n preserve $n digits from right (f.i. for 0.1234 style in MBF)
+  # no-op for $n == 0
+  # and overwrite the rest with 0's, return normalized number
+  my $x = shift; $x = $class->new($x) unless ref $x;
+  my $scale = shift || 0;
+  my $mode = shift || $rnd_mode;
+
+  # print "MBI round: $x to $scale $mode\n";
+  # -scale means what? tom? hullo? -$scale needed by MBF round, but what for?
+  return $x->bnorm() if $x->is_nan() || $x->is_zero() || $scale == 0;
+
+  # we have fewer digits than we want to scale to
+  my $len = $x->length();
+  # print "$len $scale\n";
+  return $x->bnorm() if $len < abs($scale);
+   
+  # count of 0's to pad, from left (+) or right (-): 9 - +6 => 3, or |-6| => 6
+  my ($pad,$digit_round,$digit_after);
+  $pad = $len - $scale;
+  $pad = abs($scale)+1 if $scale < 0;
+  $digit_round = '0'; $digit_round = $x->digit($pad) if $pad < $len;
+  $digit_after = '0'; $digit_after = $x->digit($pad-1) if $pad > 0;
+  # print "r $x: pos:$pad l:$len s:$scale r:$digit_round a:$digit_after\n";
+
+  # in case of 01234 we round down, for 6789 up, and only in case 5 we look
+  # closer at the remaining digits of the original $x, remember decision
+  my $round_up = 1;					# default round up
+  $round_up -- if
+    ($mode eq 'trunc')				||	# trunc by round down
+    ($digit_after =~ /[01234]/)			|| 	# round down anyway
+    ($digit_after eq '5')			&&	# 6789 => round up
+    ($x->_scan_for_nonzero($pad) == 0)		&&	# not 5000...0000
+    (
+     ($mode eq 'even') && ($digit_round =~ /[24680]/) ||
+     ($mode eq 'odd')  && ($digit_round =~ /[13579]/) ||
+     ($mode eq '+inf') && ($x->{sign} eq '-')   ||
+     ($mode eq '-inf') && ($x->{sign} eq '+')   ||
+     ($mode eq 'zero')		# round down if zero, sign adjusted below
+    );
+ 
+  # split mantissa at $scale and then pad with zeros
+  my $s5 = int($pad / 5);
+  my $i = 0;
+  while ($i < $s5)
+    {
+    $x->{value}->[$i++] = 0;				# replace with 5 x 0
+    }
+  $x->{value}->[$s5] = '00000'.$x->{value}->[$s5];	# pad with 0
+  my $rem = $pad % 5;					# so much left over
+  if ($rem > 0)
+    {
+    #print "remainder $rem\n";
+    #print "elem      $x->{value}->[$s5]\n";
+    substr($x->{value}->[$s5],-$rem,$rem) = '0' x $rem;	# overwrite with '0'
+    }
+  $x->{value}->[$s5] = int ($x->{value}->[$s5]);	# clear leading 0's
+  if ($round_up)					# what gave test above?
+    {
+    my $e = Math::BigInt->new(1);
+    $e->bneg() if $x->{sign} eq '-';
+    $pad = $len if $scale < 0;				# tlr: whack 0.51 =>1.0	
+    # modify $x in place!
+    $x->badd( $e * (Math::BigInt->new(10) ** $pad) ) if $round_up;
+    }
+  $x;
+  }
+
 ##############################################################################
 # private stuff (internal use only)
 
@@ -900,16 +1109,24 @@ sub _swap
   {
   # Overload will swap params if first one is no object ref so that the first
   # one is always an object ref. In this case, third param is true.
+  # This routine is to overcome the effect of scalar,$object creating an object
+  # of the class of this package, instead of the second param $object. This
+  # happens inside overload, when the overload section of this package is
+  # inherited by sub classes.
+  # For overload cases (and this is used only there), we need to preserve the
+  # args, hence the copy().
+  # You can override this method in a subclass, the overload section will call
+  # $object->_swap() to make sure it arrives at the proper subclass, with some
+  # exceptions like '+' and '-'.
 
   # object, (object|scalar) => preserve first and make copy
   # scalar, object	    => swapped, re-swap and create new from first
-  #                            (using class of second object)
-  my $self = shift;
-  #print "swap $_[0] $_[1] $_[2]\n";
-  my @return;
+  #                            (using class of second object, not $class!!)
+  my $self = shift;			# for override in subclass
+  #print "swap $self 0:$_[0] 1:$_[1] 2:$_[2]\n";
   if ($_[2])
     {
-    my $c = ref ($_[0] ) || $class; 	# fallback should not happen
+    my $c = ref ($_[0]) || $class; 	# fallback $class should not happen
     return ( $c->new($_[1]), $_[0] );
     }
   else
@@ -1015,13 +1232,20 @@ sub objectify
 sub import 
   {
   my $self = shift;
-  return unless @_; # do nothing for empty import lists 
+  #print "import $self @_\n";
+  for ( my $i = 0; $i < @_ ; $i++ )
+    {
+    if ( $_[$i] eq ':constant' )
+      {
+      # this rest causes overlord er load to step in
+      overload::constant integer => sub { $self->new(shift) };
+      splice @_, $i, 1; last;
+      }
+    }
   # any non :constant stuff is handled by our parent, Exporter
-  return $self->export_to_level(1,$self,@_) 
-   unless @_ == 1 and $_[0] eq ':constant';
-  #print "constant\n";
-  # the rest causes overlord er load to step in
-  overload::constant integer => sub { $self->new(@_) };
+  # even if @_ is empty, to give it a chance 
+  #$self->SUPER::import(@_);			# does not work
+  $self->export_to_level(1,$self,@_);		# need this instead
   }
 
 sub _internal 
@@ -1064,14 +1288,24 @@ sub _split
   # internal, take apart a string and return the pieces
   my $x = shift;
 
-  $$x =~ s/\s+//g;                        # strip white space (really?)
+  # pre-parse input
+  $$x =~ s/^\s+//g;			# strip white space at front
+  $$x =~ s/\s+$//g;			# strip white space at end
+  #$$x =~ s/\s+//g;			# strip white space (no longer)
   return if $$x eq "";
-  # possible inputs: 2.1234 # 0.12 # 1 # 1E1 # 2.134E1 # 434E-10
+  return if $$x !~ /^[\-\+]?\.?[0-9]/;
+
+  $$x =~ s/(\d)_(\d)/$1$2/g;		# strip underscores between digits
+  $$x =~ s/(\d)_(\d)/$1$2/g;		# do twice for 1_2_3
+  
+  # some possible inputs: 
+  # 2.1234 # 0.12        # 1 	      # 1E1 # 2.134E1 # 434E-10 # 1.02009E-2 
+  # .2 	   # 1_2_3.4_5_6 # 1.4E1_2_3  # 1e3 # +.2
 
   #print "input: '$$x' ";
   my ($m,$e) = split /[Ee]/,$$x;
   $e = '0' if !defined $e || $e eq "";
-  #print "'$m' '$e' ";
+  # print "m '$m' e '$e'\n";
   # sign,value for exponent,mantint,mantfrac
   my ($es,$ev,$mis,$miv,$mfv);
   # valid exponent?
@@ -1080,14 +1314,17 @@ sub _split
     $es = $1; $ev = $2;
     #print "'$m' '$e' e: $es $ev ";
     # valid mantissa?
+    return if $m eq '.' || $m eq '';
     my ($mi,$mf) = split /\./,$m;
-    $mf = '0' if !defined $mf;
+    $mi = '0' if !defined $mi;
+    $mi .= '0' if $mi =~ /^[\-\+]?$/;
+    $mf = '0' if !defined $mf || $mf eq '';
     if ($mi =~ /^([+-]?)0*(\d+)$/) # strip leading zeros
       {
       $mis = $1||'+'; $miv = $2;
       #print "$mis $miv";
       # valid, existing fraction part of mantissa?
-      return unless ($mf =~ /^(\d+?)0*$/);	# strip trailing zeros
+      return unless ($mf =~ /^(\d*?)0*$/);	# strip trailing zeros
       $mfv = $1;
       #print " split: $mis $miv . $mfv E $es $ev\n";
       return (\$mis,\$miv,\$mfv,\$es,\$ev);
@@ -1114,7 +1351,7 @@ sub as_number
   # it or override with their own integer conversion routine
   my $self = shift;
 
-  return Math::BigInt::bstr($self);
+  return $self->copy();
   }
 
 ##############################################################################
@@ -1176,13 +1413,6 @@ sub cmp
   return 0; # equal
   }
 
-#sub is0
-#  {
-#  # internal is_zero check
-#  my ($x,$sx) = @_;
-#  return @$x == 1 && $x->[0] == 0 && $sx eq '+';
-#  }
-
 sub add 
   {
   # (ref to int_num_array, ref to int_num_array)
@@ -1200,13 +1430,13 @@ sub add
   my $i; my $car = 0; my $j = 0;
   for $i (@$y)
     {
-    $x->[$j] -= 1e5 
-      if $car = (($x->[$j] += $i + $car) >= 1e5) ? 1 : 0; 
+    $x->[$j] -= $BASE 
+      if $car = (($x->[$j] += $i + $car) >= $BASE) ? 1 : 0; 
     $j++;
     }
   while ($car != 0)
     {
-    $x->[$j] -= 1e5 if $car = (($x->[$j] += $car) >= 1e5) ? 1 : 0; $j++;
+    $x->[$j] -= $BASE if $car = (($x->[$j] += $car) >= $BASE) ? 1 : 0; $j++;
     }
   }
 
@@ -1225,7 +1455,7 @@ sub sub
       {
       last unless defined $sy->[$j] || $car;
       #print "x: $i y: $sy->[$j] c: $car\n";
-      $i += 1e5 if $car = (($i -= ($sy->[$j] || 0) + $car) < 0); $j++;
+      $i += $BASE if $car = (($i -= ($sy->[$j] || 0) + $car) < 0); $j++;
       #print "x: $i y: $sy->[$j-1] c: $car\n";
       }
     # might leave leading zeros, so fix that
@@ -1239,7 +1469,7 @@ sub sub
       {
       last unless defined $sy->[$j] || $car;
       #print "$sy->[$j] $i $car => $sx->[$j]\n";
-      $sy->[$j] += 1e5
+      $sy->[$j] += $BASE
        if $car = (($sy->[$j] = $i-($sy->[$j]||0) - $car) < 0); 
       #print "$sy->[$j] $i $car => $sy->[$j]\n";
       $j++;
@@ -1270,7 +1500,7 @@ sub mul
       {
       $prod = $xi * $yi + ($prod[$cty] || 0) + $car;
       $prod[$cty++] =
-       $prod - ($car = int($prod * 1e-5)) * 1e5;	# see USE_MUL
+       $prod - ($car = int($prod * 1e-5)) * $BASE;	# see USE_MUL
       }
     $prod[$cty] += $car if $car; # need really to check for 0?
     $xi = shift @prod;
@@ -1294,18 +1524,18 @@ sub div
   $car = $bar = $prd = 0;
   
   my $y = [ @$yorg ];
-  if (($dd = int(1e5/($y->[-1]+1))) != 1) 
+  if (($dd = int($BASE/($y->[-1]+1))) != 1) 
     {
     for $xi (@$x) 
       {
       $xi = $xi * $dd + $car;
-      $xi -= ($car = int($xi * 1e-5)) * 1e5;	# see USE_MUL
+      $xi -= ($car = int($xi * $RBASE)) * $BASE;	# see USE_MUL
       }
     push(@$x, $car); $car = 0;
     for $yi (@$y) 
       {
       $yi = $yi * $dd + $car;
-      $yi -= ($car = int($yi * 1e-5)) * 1e5;	# see USE_MUL
+      $yi -= ($car = int($yi * $RBASE)) * $BASE;	# see USE_MUL
       }
     }
   else 
@@ -1318,15 +1548,17 @@ sub div
     {
     ($u2,$u1,$u0) = @$x[-3..-1];
     $u2 = 0 unless $u2;
-    $q = (($u0 == $v1) ? 99999 : int(($u0*1e5+$u1)/$v1));
-    --$q while ($v2*$q > ($u0*1e5+$u1-$q*$v1)*1e5+$u2);
+    print "oups v1 is 0, u0: $u0 $y->[-2] $y->[-1] l ",scalar @$y,"\n"
+     if $v1 == 0;
+    $q = (($u0 == $v1) ? 99999 : int(($u0*$BASE+$u1)/$v1));
+    --$q while ($v2*$q > ($u0*1e5+$u1-$q*$v1)*$BASE+$u2);
     if ($q)
       {
       ($car, $bar) = (0,0);
       for ($yi = 0, $xi = $#$x-$#$y-1; $yi <= $#$y; ++$yi,++$xi) 
         {
         $prd = $q * $y->[$yi] + $car;
-        $prd -= ($car = int($prd * 1e-5)) * 1e5;	# see USE_MUL
+        $prd -= ($car = int($prd * $RBASE)) * $BASE;	# see USE_MUL
 	$x->[$xi] += 1e5 if ($bar = (($x->[$xi] -= $prd + $bar) < 0));
 	}
       if ($x->[-1] < $car + $bar) 
@@ -1335,7 +1567,7 @@ sub div
 	for ($yi = 0, $xi = $#$x-$#$y-1; $yi <= $#$y; ++$yi,++$xi) 
           {
 	  $x->[$xi] -= 1e5
-	   if ($car = (($x->[$xi] += $y->[$yi] + $car) > 1e5));
+	   if ($car = (($x->[$xi] += $y->[$yi] + $car) > $BASE));
 	  }
 	}   
       }
@@ -1349,7 +1581,7 @@ sub div
       $car = 0; 
       for $xi (reverse @$x) 
         {
-        $prd = $car * 1e5 + $xi;
+        $prd = $car * $BASE + $xi;
         $car = $prd - ($tmp = int($prd / $dd)) * $dd; # see USE_MUL
         unshift(@d, $tmp);
         }
@@ -1379,15 +1611,16 @@ sub _lcm
   return $x * $ty / bgcd($x,$ty);
   }
 
-sub _gcd 
+sub _gcd_old
   { 
   # (BINT or num_str, BINT or num_str) return BINT
   # does modify first arg
-  # GCD -- Euclids algorithm Knuth Vol 2 pg 296
+  # GCD -- Euclids algorithm E, Knuth Vol 2 pg 296
   trace(@_);
  
   my $x = shift; my $ty = $class->new(shift); # preserve y
   return $x->bnan() if ($x->{sign} eq $nan) || ($ty->{sign} eq $nan);
+
   while (!$ty->is_zero())
     {
     ($x, $ty) = ($ty,bmod($x,$ty));
@@ -1395,20 +1628,89 @@ sub _gcd
   $x;
   }
 
+sub _gcd 
+  { 
+  # (BINT or num_str, BINT or num_str) return BINT
+  # does not modify arguments
+  # GCD -- Euclids algorithm, variant L (Lehmer), Knuth Vol 3 pg 347 ff
+  # unfortunately, it is slower and also seems buggy (the A=0, B=1, C=1, D=0
+  # case..)
+  trace(@_);
+ 
+  my $u = $class->new(shift); my $v = $class->new(shift);	# preserve u,v
+  return $u->bnan() if ($u->{sign} eq $nan) || ($v->{sign} eq $nan);
+  
+  $u->babs(); $v->babs();		# Euclid is valid for |u| and |v|
+
+  my ($U,$V,$A,$B,$C,$D,$T,$Q);		# single precision variables
+  my ($t);				# multiprecision variables
+
+  while ($v > $BASE)
+    {
+    #sleep 1;
+    ($u,$v) = ($v,$u) if ($u < $v); 		# make sure that u >= v
+    #print "gcd: $u $v\n";
+    # step L1, initialize
+    $A = 1; $B = 0; $C = 0; $D = 1;
+    $U = $u->{value}->[-1];			# leading digits of u
+    $V = $v->{value}->[-1];			# leading digits of v
+      
+    # step L2, test quotient
+    if (($V + $C != 0) && ($V + $D != 0))	# div by zero => go to L4
+      {
+      $Q = int(($U + $A)/($V + $C));		# quotient
+      #print "L1 A=$A B=$B C=$C D=$D U=$U V=$V Q=$Q\n";
+      # do L3? (div by zero => go to L4)
+      while ($Q == int(($U + $B)/($V + $D)))
+        {
+        # step L3, emulate Euclid
+        #print "L3a A=$A B=$B C=$C D=$D U=$U V=$V Q=$Q\n";
+        $T = $A - $Q*$C; $A = $C; $C = $T;
+        $T = $B - $Q*$D; $B = $D; $D = $T;
+        $T = $U - $Q*$V; $U = $V; $V = $T;
+        last if ($V + $D == 0) || ($V + $C == 0);	# div by zero => L4
+        $Q = int(($U + $A)/($V + $C));	# quotient for next test
+        #print "L3b A=$A B=$B C=$C D=$D U=$U V=$V Q=$Q\n";
+        }
+      }
+    # step L4, multiprecision step
+    # was if ($B == 0)
+    # in case A = 0, B = 1, C = 0 and D = 1, this case would simple swap u & v
+    # and loop endless. Not sure why this happens, Knuth does not make a
+    # remark about this special case. bug?
+    if (($B == 0) || (($A == 0) && ($C == 1) && ($D == 0)))
+      {
+      #print "L4b1: u=$u v=$v\n";
+      ($u,$v) = ($v,bmod($u,$v)); 
+      #$t = $u % $v; $u = $v->copy(); $v = $t;
+      #print "L4b12: u=$u v=$v\n";
+      }
+    else
+      {
+      #print "L4b: $u $v $A $B $C $D\n";
+      $t = $A*$u + $B*$v; $v *= $D; $v += $C*$u; $u = $t;
+      #print "L4b2: $u $v\n";
+      }
+    } # back to L1
+
+  return _gcd_old($u,$v) if $v != 1;	# v too small
+  return $v;				# 1
+  }
+
 ###############################################################################
 # this method return 0 if the object can be modified, or 1 for not
 # We use a fast use constant statement here, to avoid costly calls. Subclasses
 # may override it with special code (f.i. Math::BigInt::Constant does so)
 
-#use constant modify => 0;
+use constant modify => 0;
 
-sub modify
-  {
-  my $self = shift;
-  my $method = shift;
+#sub modify
+#  {
+#  my $self = shift;
+#  my $method = shift;
 #  print "original $self modify by $method\n";
-  return 0; # $self;
-  }
+#  return 0; # $self;
+#  }
 
 1;
 __END__
@@ -1467,21 +1769,31 @@ Math::BigInt - Arbitrary size integer math package
   $x->bior($y);			# bitwise inclusive or
   $x->bxor($y);			# bitwise exclusive or
   $x->bnot();			# bitwise not (two's complement)
+
+  $x->bround($N);               # accuracy: preserver $N digits
+  $x->bfround($N);              # round to $Nth digit, no-op for BigInts
+  $x->bsqrt();			# calculate square-root
   
   # The following do not modify their arguments:
 
   bgcd(@values);		# greatest common divisor
   blcm(@values);		# lowest common multiplicator
   
-  $x->bstr();			# return normalized string
+  $x->bstr();			# normalized string
+  $x->bsstr();			# normalized string in scientific notation
   $x->length();			# return number of digits in number
+  ($x,$f) = $x->length();	# length of number and length of fraction part
+
+  $x->exponent();               # return exponent as BigInt
+  $x->mantissa();               # return mantissa as BigInt
+  $x->parts();                  # return (mantissa,exponent) as BigInt
 
 =head1 DESCRIPTION
 
 All operators (inlcuding basic math operations) are overloaded if you
 declare your big integers as
 
-  $i = new Math::BigInt '123 456 789 123 456 789';
+  $i = new Math::BigInt '123_456_789_123_456_789';
 
 Operations with overloaded operators preserve the arguments which is
 exactly what you expect.
@@ -1494,13 +1806,16 @@ Big integer values are strings of the form C</^[+-]\d+$/> with leading
 zeros suppressed.
 
    '-0'                            canonical value '-0', normalized '0'
-   '   -123 123 123'               canonical value '-123123123'
-   '1 23 456 7890'                 canonical value '1234567890'
+   '   -123_123_123'               canonical value '-123123123'
+   '1_23_456_7890'                 canonical value '1234567890'
 
 =item Input
 
 Input values to these routines may be either Math::BigInt objects or
-strings of the form C</^\s*[+-]?[\d\s]+\.?[\d\s]*E?[+-]?[\d\s]*$/>.
+strings of the form C</^\s*[+-]?[\d]+\.?[\d]*E?[+-]?[\d]*$/>.
+
+You can include underscores between any two digits, but not more than one
+underscore between two digits.
 
 This means integer values like 1.01E2 or even 1000E-2 are also accepted.
 Non integer values result in NaN.
@@ -1522,12 +1837,66 @@ return either undef, <0, 0 or >0 and are suited for sort.
 
 =back
 
+=head2 Rounding
+
+Only C<bround()> is defined for BigInts, for further details of rounding see
+L<Math::BigFloat>.
+
+=over 2
+
+=item bfround ( +$scale ) rounds to the $scale'th place left from the '.'
+
+=item bround  ( +$scale ) preserves accuracy to $scale digits from the left (aka significant digits) and paddes the number with zeros
+
+=item bround  ( -$scale ) preserves accuracy to $scale digits from the right (aka significant digits) and paddes the number with zeros. This is used by Math::BigFloat.
+
+=back
+
+C<bfround()> does nothing in case of negative C<$scale>. Both C<bround()> and
+C<bfround()> are a no-ops for a scale of 0.
+
+All rounding functions take as a second parameter a rounding mode from one of
+the following: 'even', 'odd', '+inf', '-inf', 'zero' or 'trunc'.
+
+The default is 'even'. By using C<< Math::BigInt->round_mode($rnd_mode); >>
+you can get and set the default round mode for subsequent rounding. 
+
+The second parameter to the round functions than overrides the default
+temporarily.
+
+=head2 Internals
+
 Actual math is done in an internal format consisting of an array of
 elements of base 100000 digits with the least significant digit first.
 The sign C</^[+-]$/> is stored separately. The string 'NaN' is used to 
 represent the result when input arguments are not numbers, as well as 
 the result of dividing by zero.
 
+You sould neither care nor depend on the internal represantation, it might
+change without notice. Use only method calls like C<< $x->sign(); >> instead
+relying on the internal hash keys like in C<< $x->{sign}; >>. 
+
+=head2 mantissa(), exponent() and parts()
+
+C<mantissa()> and C<exponent()> return the said parts of the BigInt such
+that:
+
+        $m = $x->mantissa();
+        $e = $x->exponent();
+        $y = $m * ( 10 ** $e );
+        print "ok\n" if $x == $y;
+
+C<($m,$e) = $x->parts()> is just a shortcut that gives you both of them in one
+go. Both the returned mantissa and exponent do have a sign.
+
+Currently, for BigInts C<$e> will be always 0, except for NaN where it will be
+NaN and for $x == 0, then it will be 1 (to be compatible with Math::BigFlaot's
+internal representation of a zero as C<0E1>).
+
+C<$m> will always be a copy of the original number. The relation between $e
+and $m might change in the future, but will be always equivalent in a
+numerical sense, e.g. $m might get minized.
+ 
 =head1 EXAMPLES
  
   use Math::BigInt qw(bstr bint);
@@ -1547,6 +1916,7 @@ the result of dividing by zero.
   $x--;                              	# BigInt "-1"
   $x = Math::BigInt->badd(4,5)		# BigInt "9"
   $x = Math::BigInt::badd(4,5)		# BigInt "9"
+  print $x->bsstr();			# 9e+0
 
 =head1 Autocreating constants
 
@@ -1580,13 +1950,13 @@ Using the form $x += $y; etc over $x = $x + $y is faster, since a copy of $x
 must be made in the second case. For long numbers, the copy can eat up to 20%
 of the work (in case of addition/subtraction, less for
 multiplication/division). If $y is very small compared to $x, the form
-$x += $y is MUCH faster than $x = $x + $y since the copy of $x takes more
-time then the actual addition.
+$x += $y is MUCH faster than $x = $x + $y since making the copy of $x takes
+more time then the actual addition.
 
 The new version of this module is slower on new(), bstr() and numify(). Some
 operations may be slower for small numbers, but are significantly faster for
-big numbers. Other operations are now constant (O(1), bneg(), babs() etc),
-instead of O(N) and thus nearly always take much less time.
+big numbers. Other operations are now constant (O(1), like bneg(), babs()
+etc), instead of O(N) and thus nearly always take much less time.
 
 For more benchmark results see http://bloodgate.com/perl/benchmarks.html
 
@@ -1611,7 +1981,7 @@ known to be troublesome:
 
 =over 1
 
-=item stringify, bstr()
+=item stringify, bstr(), bsstr() and 'cmp'
 
 Both stringify and bstr() now drop the leading '+'. The old code would return
 '+3', the new returns '3'. This is to be consistent with Perl and to make
@@ -1645,6 +2015,21 @@ Additionally, the following still works:
 	print "$x == 9" if $x == 9;
 	print "$x == 9" if $x == 3*3;
 
+There is now a C<bsstr()> method to get the string in scientific notation aka
+C<1e+2> instead of C<100>. Be advised that overloaded 'eq' always uses bstr()
+for comparisation, but Perl will represent some numbers as 100 and others
+as 1e+308. If in doubt, convert both arguments to Math::BigInt before doing eq:
+
+	use Test;
+        BEGIN { plan tests => 3 }
+	use Math::BigInt;
+
+	$x = Math::BigInt->new('1e56'); $y = 1e56;
+	ok ($x,$y);			# will fail
+	ok ($x->bsstr(),$y);		# okay
+	$y = Math::BigInt->new($y);
+	ok ($x,$y);			# okay
+
 =item bdiv
 
 The following will probably not do what you expect:
@@ -1684,6 +2069,26 @@ will do what the underlying C thinks is right and this is different for each
 system. If you need BigInt's behaving exactly like Perl's 'use integer', bug
 the author to implement it ;)
 
+=item Modifying and =
+
+Beware of:
+
+        $x = Math::BigFloat->new(5);
+        $y = $x;
+
+It will not do what you think, e.g. making a copy of $x. Instead it just makes
+a second reference to the B<same> object and stores it in $y. Thus anything
+that modifies $x will modify $y, and vice versa.
+
+        $x->bmul(2);
+        print "$x, $y\n";       # prints '10, 10'
+
+If you want a true copy of $x, use:
+
+        $y = $x->copy();
+
+See also the documentation in L<overload> regarding C<=>.
+
 =item bpow
 
 C<bpow()> now modifies the first argument, unlike the old code which left
@@ -1763,12 +2168,33 @@ Scalar values are a bit different, since:
 	$float = 2 + $mbf;
 	$float = $mbf + 2;
 
-will both result in the proper type due to the way overload works.
+will both result in the proper type due to the way overloaded math works.
 
 This section also applies to other overloaded math packages, like Math::String.
 
+=item bsqrt()
+
+C<bsqrt()> works only good if the result is an big integer, e.g. the square
+root of 144 is 12, but from 12 the square root is 3, regardless of rounding
+mode.
+
+If you want a better approximation of the square root, then use:
+
+	$x = Math::BigFloat->new(12);
+	Math::BigFloat::precision(0);
+	Math::BigFloat::round_mode('even');
+	print $x->copy->bsqrt(),"\n";		# prints 4
+
+	Math::BigFloat::precision(2);
+	print $x->bsqrt(),"\n";			# prints 3.46
+
 =back
 
+=head1 LICENSE
+
+This program is free software; you may redistribute it and/or modify it under
+the same terms as Perl itself.
+
 =head1 AUTHORS
 
 Original code by Mark Biggar, overloaded interface by Ilya Zakharevich.
